#!/bin/bash

SCRIPT_NAME=[[`basename "$0"`]]

# This script expects the (full) path of libreadline.a (or the string "-lreadline").
# If the arg is "-lreadline", the script simply exits with code 0.
# It prints out the (full) path of (hopefully) the corresponding readline.h.

# taken from StackExchange 256434
is_absolute()
{
    case "$1" in
	///* | //) true;;
	//*) false;;
	/*) true;;
	*) false;;
    esac
}


if [ $# -ne 1 ]
then
  echo "ERROR:: expected 1 arg (full path of libreadline)   $SCRIPT_NAME"  > /dev/stderr
  exit 1
fi

READLINE_LIB="$1"

# If READLINE is in std directory, assume header is too.  Output nothing.
if [ "X$READLINE_LIB" = "X-lreadline" ]
then
  exit 0
fi

# The following is a cryptic if...then block
is_absolute "$READLINE_LIB" ||
(
  echo "ERROR: arg is not an absolute path: \"$READLINE_LIB\"   $SCRIPT_NAME"  > /dev/stderr
  exit 1
)

if [ \! -f "$READLINE_LIB" -o \! -r "$READLINE_LIB" ]
then
  echo "ERROR: specified READLINE library is unreadable $READLINE_LIB   $SCRIPT_NAME"  > /dev/stderr
  exit 1
fi


READLINE_LIB_DIR=`dirname "$READLINE_LIB"`
READLINE_LIB_DIR_DIR=`dirname "$READLINE_LIB_DIR"`
READLINE_HDR_DIR="$READLINE_LIB_DIR_DIR/include/readline"
READLINE_HDR="$READLINE_HDR_DIR/readline.h"

if [ -f "$READLINE_HDR" -a -r "$READLINE_HDR" ]
then
  # We've found a plausible readline.h.
  echo "$READLINE_HDR"
  exit 0
fi

echo "ERROR: Trouble reading READLINE header file $READLINE_HDR   $SCRIPT_NAME"  > /dev/stderr
exit 2
