//   Copyright (c)  2015  John Abbott

//   This file is part of the source of CoCoALib, the CoCoA Library.

//   CoCoALib is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.

//   CoCoALib is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.

//   You should have received a copy of the GNU General Public License
//   along with CoCoALib.  If not, see <http://www.gnu.org/licenses/>.


#include "CoCoA/interrupt.H"
#include "CoCoA/verbose.H"
#include "CoCoA/GlobalManager.H"

#include <csignal>
using std::signal;
#include <iostream>
using std::ostream;

namespace CoCoA
{

  const char* const InterruptReceived::ourExceptionMesg = "External interrupt";

  namespace // anonymous
  {
    volatile sig_atomic_t SignalReceived = 0; // GLOBAL VARIABLE!!!

    void SetSignalReceived(int sig)
    {
      SignalReceived = sig;
    }
    
    void AnnounceInterruption()
    {
      VerboseLog VERBOSE("intr");
      VERBOSE(1) << std::endl;
      VERBOSE(1) << "-------------------------" << std::endl;
      VERBOSE(1) << ">>> CoCoA interrupted <<<" << std::endl;
      VERBOSE(1) << "-------------------------" << std::endl;
    }

  } // end of anonymous namespace


  InterruptReceived::~InterruptReceived()
  {
    ResetInterruptSignalReceived();
  }


  void CheckForInterrupt(const char* const context)
  {
    const int SignalReceived = InterruptSignalReceived();
    if (!SignalReceived) return;
    AnnounceInterruption();
    throw InterruptReceived(SignalReceived, context);
  }

  void CheckForInterrupt(const std::string& context)
  {
    const int SignalReceived = InterruptSignalReceived();
    if (!SignalReceived) return;
    AnnounceInterruption();
    throw InterruptReceived(SignalReceived, context);
  }


  std::ostream& operator<<(std::ostream& out, const InterruptReceived& intr)
  {
    if (!out) return out;  // short-cut for bad ostreams
    out << "CoCoA::InterruptReceived(signal=" << TriggeredBySignal(intr) << ", context=" << context(intr) << ")";
    return out;
  }


  void SignalInterruptsCoCoA(int sig)
  {
    signal(sig, SetSignalReceived);
    GlobalManager::ptr("SignalInterruptsCoCoA")->myNewInterruptSignalReceivedPtr(&SignalReceived);
  }

} // end of namespace CoCoA


// RCS header/log in the next few lines
// $Header: /Volumes/Home_1/cocoa/cvs-repository/CoCoALib-0.99/src/AlgebraicCore/interrupt.C,v 1.5 2016/11/18 18:12:09 abbott Exp $
// $Log: interrupt.C,v $
// Revision 1.5  2016/11/18 18:12:09  abbott
// Summary: InterruptReceived now stores signal; added new fns TriggeredBySignal, SignalInterruptsCoCoA
//
// Revision 1.4  2016/11/11 14:15:34  abbott
// Summary: Added short-cut to operator<< when ostream is in bad state
//
// Revision 1.3  2015/06/26 14:58:03  abbott
// Summary: Now InterruptReceived derives from CoCoA::exception; CheckForInterrupt requires context string.
// Author: JAA
//
// Revision 1.2  2015/06/25 16:09:31  abbott
// Summary: InterruptReceived ctor now requires string arg; added printing.
// Author: JAA
//
// Revision 1.1  2015/05/20 14:44:58  abbott
// Summary: New fns for responding to interrupts
// Author: JAA
//
//
